/*                                 titecrm.sas                                  */
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Time-to-Event Continual Reassessment Method, Cheung & Chappell (2000)       */
/*                                                                              */
/*                                                                              */
/*  Version 7:  Simulation reports number of trials that stop early due to      */
/*                  excessive early toxicity                                    */
/*              Added macro variable NRUNIN to force minimum number of          */
/*                  patients                                                    */
/*              Added option ADM_DOSE to select "nearest dose to target" in     */
/*                  addition to "highest dose less than target"                 */
/*                                                                              */
/*  Version 6:  Improved interim result reporting                               */
/*              Rationalized specification of nonconstant hazard function       */
/*              Simplified data input                                           */
/*              Adaptive domain of numerical integration                        */
/*              Many bug fixes                                                  */
/*                                                                              */
/*  Version 5:  Optionally retain incomplete patients                           */
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  This program requires SAS 9, and has been tested on Fedora Core 8.          */
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  See macro invocations at end of this file for single trial (including       */
/*  interim) analysis (README file mode 1) and simulations for determining      */
/*  operating charactersitics (README file mode 2).                             */
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  COPYRIGHT  2007-2008                                                        */
/*  THE REGENTS OF THE UNIVERSITY OF MICHIGAN                                   */
/*  ALL RIGHTS RESERVED                                                         */
/*                                                                              */
/*  PERMISSION IS GRANTED TO USE, COPY, AND CREATE DERIVATIVE WORKS OF THIS     */
/*  SOFTWARE FOR NONCOMMERCIAL EDUCATION AND RESEARCH PURPOSES, SO LONG AS NO   */
/*  FEE IS CHARGED, AND SO LONG AS THE COPYRIGHT NOTICE ABOVE, THIS GRANT OF    */
/*  PERMISSION, AND THE DISCLAIMER BELOW APPEAR IN ALL COPIES MADE; AND SO LONG */
/*  AS THE NAME OF THE UNIVERSITY OF MICHIGAN IS NOT USED IN ANY ADVERTISING OR */
/*  PUBLICITY PERTAINING TO THE USE OF THIS SOFTWARE WITHOUT SPECIFIC, WRITTEN  */
/*  PRIOR AUTHORIZATION. PERMISSION TO DISTRIBUTE THIS SOFTWARE OR DERIVATIVE   */
/*  WORKS OF THIS SOFTWARE IS NOT GRANTED.                                      */
/*                                                                              */
/*  THIS SOFTWARE IS PROVIDED AS IS, WITHOUT REPRESENTATION FROM THE UNIVERSITY */
/*  OF MICHIGAN AS TO ITS FITNESS FOR ANY PURPOSE, AND WITHOUT WARRANTY BY THE  */
/*  UNIVERSITY OF MICHIGAN OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING    */
/*  WITHOUT LIMITATION THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS    */
/*  FOR A PARTICULAR PURPOSE. THE REGENTS OF THE UNIVERSITY OF MICHIGAN SHALL   */
/*  NOT BE LIABLE FOR ANY DAMAGES, INCLUDING SPECIAL, INDIRECT, INCIDENTAL, OR  */
/*  CONSEQUENTIAL DAMAGES, WITH RESPECT TO ANY CLAIM ARISING OUT OF OR IN       */
/*  CONNECTION WITH THE USE OF THE SOFTWARE, EVEN IF IT HAS BEEN OR IS          */
/*  HEREAFTER ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.                       */
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Global macro variable definitions                                           */
/*                                                                              */
/*  All runs:                                                                   */
/*                                                                              */
/*    adm_dose    1=Largest P(DLT) le TARGET                                    */
/*                2=P(DLT) closest to TARGET                                    */
/*    adm_marg    Allowable excess of (PDLT) above TARGET                       */
/*    dateflag    Method for setting TODAY:                                     */
/*                0=Current system date  (i.e., right now)                      */
/*                1=Last date relevant to trial (i.e., at end of trial)         */
/*                2=Value in SOMEDATE                                           */
/*    debug       Debug flag, ON=1                                              */
/*    dosefile    Dose descriptor file name                                     */
/*    dosejump    Number of dose level increases allowed between consecutive    */
/*                patients                                                      */
/*    dtmodel     Dose-toxicity model, logisitic or Lyman                       */
/*    gamma       Gamma parameter for Lyman model                               */
/*    iflag       0=Patients who do not complete observation period become      */
/*                  unevaluable                                                 */
/*                1=Patients who do not complete observation period are         */
/*                  evaluable and downweighted (if response=2)                  */
/*    level1      First dose level for trial                                    */
/*    nproceed    Number of patients needed to complete before dose escalation  */
/*    nrunin      Number of enrolled patients before stopping rule is checked   */
/*    obsdays     Number of days patient is followed for toxicity               */
/*    prior       Prior distribution, normal, beta or exp                       */
/*    priora      alpha of beta prior                                           */
/*    priorb      beta of beta prior                                            */
/*    priorstd    STD of normal prior                                           */
/*    rscheme     Restriction scheme for dose-escalation                        */
/*                0=No restriction, use target dose                             */
/*                1=Allow only dosejump levels between patients                 */
/*                2=Require NPROCEED complete patients before escalation        */
/*                3=Reuqire NPROCEED observation periods before escalation      */
/*                12=1+2                                                        */
/*                13=1+3                                                        */
/*    simflag     0=Single trial, 1=Simulate                                    */
/*    somedate    Fixed date (since 1/1/1960)                                   */
/*    target      Target toxicity probability                                   */
/*    td50        TD50 parameter for Lyman model                                */
/*    titeflag    TITE-CRM flag =1 for TITE-CRM, 0 for CRM                      */
/*    vwdays      d1...dk,obsdays for variable weighting                        */
/*    vwflag      0=constant weighting throughout trial                         */
/*                1=variable weighting                                          */
/*    vwwts       w1...wk-1,1 for variable weighting                            */
/*                                                                              */
/*  Simulations only:                                                           */
/*                                                                              */
/*    clearsim    Clear previous simulations                                    */
/*                0=No                                                          */
/*                1=Yes                                                         */
/*    maxsub      Total number of subjects recutited if varcruit=0              */
/*    nsim        Number of simulated trials                                    */
/*    outdir      Directory for output SAS Data Files PATIENT and TRIAL         */
/*    periods     Periods (in days) for rectimes if varcruit=1                  */
/*    rectime     Mean interarrival time between patients if varcruit=0         */
/*    rectimes    Mean interarrival times between patients if varcruit=1        */
/*    seed1       Random number seed for simulations                            */
/*    simtag      ID tag for simulation runs                                    */
/*    urdays      d1...dk,obsdays for variable weighting                        */
/*    urflag      Variable (non-uniform) hazard                                 */
/*                0=No, use uniform hazard                                      */
/*                1=Yes, use non-uniform hazard                                 */
/*    urwts       w1...wk-1,1 for variable weighting                            */
/*    varcruit    Variable recruitment                                          */
/*                0=No, use macro variables maxsub and rectime                  */
/*                1=Yes, use macro variables periods and rectimes               */
/*                                                                              */
/*  Single trial runs only:                                                     */
/*                                                                              */
/*    npat        Number of evaluable patients                                  */
/*    patfile     Patient data full file name                                   */
/*    patform     Format type of patient data                                   */
/*                1=Dose levels (e.g., 1,2,3...) are specified                  */
/*                2=Dose names (e.g., 25 mg/m2) are specified                   */
/*    title1      Top level title for printouts                                 */
/*                                                                              */
/*  Internally set:                                                             */
/*                                                                              */
/*    nlevel      Number of dose levels                                         */
/*    readerr     Patient data read error flag, 0=No Error, 1=Error             */
/*    today       Date of calculation                                           */
/*                                                                              */
/********************************************************************************/

/********************************************************************************/
/*                                                                              */
/*  DEFAULTS is a place to put macro defaults, so the program will at least run */
/*                                                                              */
/********************************************************************************/


%macro defaults;

       options ls=85;

       %global adm_dose
               adm_marg
               clearsim
               dateflag
               debug
               dosefile
               dosejump
               dtmodel
               gamma
               iflag
               level1
               maxsub
               nlevel
               npat
               nproceed
               nrunin
               nsim
               obsdays
               outdir
               patfile
               patform
               periods
               prior
               priorstd
               priora
               priorb
               readerr
               rectime
               rectimes
               rscheme
               seed1
               simflag
               simtag
               somedate
               srcdate
               target
               td50
               titeflag
               title1
               today
               urdays
               urflag
               urn
               urwts
               varcruit
               vwdays
               vwflag
               vwn
               vwwts
           ;


*------Date of this source code;

       %let srcdate=10DEC2008;

*------Debug flag is ON (0=OFF);

       %let debug=1;

*------Date of trial assessment is the current system date;

       %let dateflag=0;

*------Only jump one dose level between patients;

       %let dosejump=1;

*------Standard logistic dose-toxicity model;

       %let dtmodel=logistic;

*------Do TITE-CRM;

       %let titeflag=1;

*------Start at the first dose level;

       %let level1=2;

*------Use a normal prior with a mean of 1 (always) and a standard
       deviation of 0.3;

       %let prior=normal;
       %let priorstd=0.3;

*------Require 1 completed patient without DLT before escalation;

       %let nproceed=1;
       %let rscheme=12;

*------Set admissable dose to largest le TARGET, ADM_MARG turned off;

       %let adm_dose=1;
       %let adm_marg=1.0;

*------If doing estimation based on data (not a simulation), assume 2nd
       column of PATIENT data file has dose name tags (see PATDATA
       macro).  1st column of DOSE data file always has dose name
       tags;

       %let patform=2;

*------Assume constant weighting across observation period;

       %let vwflag=0;

*------Enroll NRUNIN patients before checking stopping rule;

       %let nrunin=1;

*------Clear previous simulations;

       %let clearsim=1;

*------Assume uniform risk across observation period (simulations only);

       %let urflag=0;

*------Assume constant recuitment rate;

       %let varcruit=0;

*------Patients without DLT removed from the trial early are NOT counted;

       %let iflag=0;

*------Title for printouts;

       %let title1=Example Trial;

*------Current directory for simulation output;

       %let outdir=%str(.);

       %mend defaults;


/********************************************************************************/
/*                                                                              */
/*  DEFDOSE  creates the SAS data file defining the initial toxicity            */
/*           probability for each dose.                                         */
/*                                                                              */
/*  INPUT:                                                                      */
/*       Macro variables:                                                       */
/*            DOSEFILE  (input data set name, with extension)                   */
/*            DEBUG     (Debug flag)                                            */
/*                                                                              */
/*       Input data set:      DOSEFILE        (Tab delimited)                   */
/*            Dose Tag        Required, Used for Identification only, up to 20  */
/*                            characters                                        */
/*            Initial P(Tox)  Required, also determines dose ordering           */
/*            True P(Tox)     Simulations Only                                  */
/*                                                                              */
/*  OUTPUT:                                                                     */
/*       SAS Data File:       DOSES                                             */
/*       Macro variable:      NLEVEL                                            */
/*                                                                              */
/********************************************************************************/

%macro defdose;

       %if &debug=1 %then %do;
           %put DEFDOSE (ENTER) DOSEFILE &dosefile;
           %end;

*------Read DOSEFILE, only first three fields;

       filename doses "&dosefile";

       data defdose1;
            length dose $ 20;
            retain nlevel 0;
            infile doses eof=lastline
                         dsd
                         missover
                         delimiter='09'x;
            input dose p truep;

            if (p gt .) then do
               t=log(p/(1-p))-3;
               nlevel+1;
               output;
               end;

            return;

         lastline:  call symput('nlevel',left(put(nlevel,2.)));
            stop;

            keep dose p truep t;
            run;

        proc sort data=defdose1;
             by p;
             run;

        data doses;
             retain level 0;
             set defdose1;
             level+1;
             run;

       %if &debug=0 %then %do;

           proc datasets library=work;
                delete defdose1 / memtype=data;
                run;
                quit;

           %end;

       %if &debug=1 %then %do;

           %put DEFDOSE (EXIT) NLEVEL &nlevel;

           data _null_;
                set doses;
                put 'LEVEL ' level  '   DOSE ' dose '   P ' p @;
                if (truep gt .) then put '   TRUE P ' truep @;
                put;
                run;

           %end;

       %mend defdose;

/********************************************************************************/
/*                                                                              */
/*  DDAY     sets the macro variable TODAY according to the input parameters    */
/*                                                                              */
/*  INPUT:                                                                      */
/*       Macro variables:    DEBUG                                              */
/*                           dateflag 0=Current system date                     */
/*                                    1=Last DATEON in ARRIVE                   */
/*                                    2=Value in SOMEDATE in date9 format,      */
/*                                   e.g., 01FEB1999                            */
/*       SAS Data file:      ARRIVE                                             */
/*                                                                              */
/*  OUTPUT:                                                                     */
/*       Macro variables:    TODAY                                              */
/*                                                                              */
/********************************************************************************/

%macro dday;

       %if &debug=1 %then %do;

           %put DDAY (ENTER) DATEFLAG &dateflag;
           %put DDAY (ENTER) SOMEDATE &somedate;
           %put DDAY (ENTER) OBSDAYS &obsdays;

           %end;

       %if &dateflag=0 %then %do;

           data _null_;
                call symput('today',left(put(date(),8.)));
                stop;
                run;

           %end;

       %if &dateflag=1 & &simflag=1 %then %do;

           %put ATTEMPT TO SET DATE FROM SIMULATED DATA;
           %put  MUST USE DATEFLAG=1 WITH INPUT DATA SET;

           %end;


       %if &dateflag=1 & &simflag=0 %then %do;

          filename patient "&patfile";

           data _null_;
                length dose $ 20;
                retain lastdate -1E6 obsdays .;
                informat dateon respdate date9.;
                infile patient
                       missover
                       dsd
                       delimiter='09'x
                       eof=lastline;
                input id dose dateon response respdate;
                if (_n_ eq 1) then obsdays=input(symget('obsdays'),3.);
                if (respdate gt lastdate) then lastdate=respdate;
                if (dateon+obsdays gt lastdate) then lastdate=dateon+obsdays;
                return;
              lastline:
                call symput('today',left(put(lastdate,8.)));
                stop;
                run;

           %end;


       %if &dateflag=2 %then %do;

            data _null_;
                 somedate=input(symget('somedate'),date9.);
                 call symput('today',left(put(somedate,8.)));
                 stop;
                 run;

            %end;


       %if &debug=1 %then %do;

           %put DDAY (EXIT) TODAY &today;

           %end;

      %mend dday;


/********************************************************************************/
/*                                                                              */
/*  PATDATA  reads in a text file describing the events in trial and            */
/*           builds the data set ARRIVE containing trial status used by         */
/*           other TITE-CRM macros to estimate toxicity and assign doses.       */
/*                                                                              */
/*                                                                              */
/*  INPUT:                                                                      */
/*    Macro variables:    DEBUG     detail flag (0/1)                           */
/*                        PATFILE   full input file name (include extension),   */
/*                                  assumed to be in current directory,         */
/*                                  tab-delimited:                              */
/*                        PATFORM   Format of input file (tab delimited)        */
/*                                  1 or 2, determines dose identifier          */
/*                                  Columns of PATFILE are:                     */
/*                                      1=Patient ID                            */
/*                                      2=Level (Patform=1) =1,2,3,4...         */
/*                                        Dose (Patform=2) or                   */
/*                                      3=On-study date (date9:  DDMMMYYYY)     */
/*                                      4=Response:                             */
/*                                          0=No DLT so far                     */
/*                                          1=DLT on date to follow             */
/*                                          2=Not evaluable as of date to       */
/*                                            follow                            */
/*                                          3=Never evaluable, always discard   */
/*                                      5=Response date (date 9:  DDMMMYYYY)    */
/*                                  if columns 4 and 5 are blank, then the      */
/*                                  response is set to 0 (No DLT) as of         */
/*                                  TODAY.  If the response is set to 0 and the */
/*                                  date is less than TODAY, the response is    */
/*                                  to be missing as of TODAY.                  */
/*                                                                              */
/*                                    Example:                                  */
/*                                       ABC 2 03JAN2001 0 05MAR2001 0          */
/*                                                                              */
/*                                                                              */
/*  OUTPUT:                                                                     */
/*       SAS Data File:       ARRIVE                                            */
/*       Macro variables:     READERR   0=No Error                              */
/*                                      1=Patient Data Read Error               */
/*                                                                              */
/********************************************************************************/

%macro patdata;

       %if &debug=1 %then %do;
           %put PATDATA (ENTER) PATFILE &patfile;
           %put PATDATA (ENTER) PATFORM &patform;
           %end;


       proc sort data=doses;
            by dose;
            run;

       filename patient "&patfile";

       %if &patform=1 %then %let vlist=%str(id level dateon response respdate);
       %if &patform=2 %then %let vlist=%str(id dose dateon response respdate);


           data patdata1;
                %if &patform=1 %then %do;
                    length id $ 20;
                   %end;
                %if &patform=2 %then %do;
                    length id dose $ 20;
                   %end;
                retain today .;
                informat dateon respdate date9.;
                format dateon respdate date9.;
                infile patient missover dsd
                       delimiter='09'x;

               input &vlist;

                if (_n_ eq 1) then do;
                    today=input(symget('today'),8.);
                    end;

                if (dateon le today) then output;
                drop today;

                run;

           %if &patform=1 %then %let sortvar=level;
           %if &patform=2 %then %let sortvar=dose;


            proc sort data=patdata1;
                 by &sortvar;
                 run;

            proc sort data=doses;
                 by &sortvar;
                 run;

            data patdata2;
                 merge patdata1(in=in1) doses(keep=level dose);
                 by &sortvar;
                 if in1;
                 run;

           proc sort data=patdata2;
                by dateon;
                run;

*----------Build master patient data set (ALL) and working patient data set
           (ARRIVE);

           data all(keep=id dose level dateon respdate response)
                   arrive(keep=id level dateon respdate response)
                   nocount(keep=id dose level dateon respdate response);
                retain ngood 0 titeflag .;
                set patdata2 end=last;

                if (_n_ eq 1) then do;
                    iflag=input(symget('iflag'),1.);
                    titeflag=input(symget('titeflag'),1.);
                    today=input(symget('today'),8.);
                    obsdays=input(symget('obsdays'),3.);
                    end;

                output all;

*---------------Use the observation if CRM only and complete, or
                TITECRM and evaluable (if iflag=0) and
                ever evaluable (response lt 3);

                if ((titeflag eq 0) and (response eq 0) and
                       (respdate lt dateon+obsdays)) or
                       ((titeflag eq 1) and (response eq 2) and (iflag eq 0)) or
                       (response eq 3) or
                       (level eq .) then do;
                   output nocount;
                   end;

                else do;
                   if (response eq 0) and (respdate lt today) then
                       respdate=min(dateon+obsdays,today);

                   output arrive;
                   ngood+1;
                   end;

                if (last) then do;
                    call symput('npat',trim(left(put(ngood,3.0))));
                    end;

                run;

        proc sort data=arrive;
             by dateon;
             run;

        proc sort data=all;
             by dateon;
             run;

       %if &debug=0 %then %do;
           proc datasets library=work;
                delete patdata1 patdata2 / memtype=data;
                run;
                quit;
           %end;

       %if &debug=1 %then %do;
           %put PATDATA (EXIT) MAXSUB &maxsub;
           %end;

       %mend patdata;


/********************************************************************************/
/*                                                                              */
/* SIMTITE does the CRM (TITEFLAG=0) or TITE-CRM (TITEFLAG=1) calculations.     */
/*         If SIMFLAG=1, it simulates CRM or TITE-CRM trials according to the   */
/*         trial parameters set by the file DOSEFILE and the trial and          */
/*         simulation characteristic parameters.  If SIMFLAG=0, it calculates   */
/*         the probability of toxicity at each of the dose levels.              */
/*                                                                              */
/********************************************************************************/
/*  adm_dose    1=Largest P(DLT) le TARGET                                      */
/*              2=P(DLT) closest to TARGET                                      */
/*  adm_marg    Allowable excess of (PDLT) above TARGET                         */
/*  debug       Debug flag, ON=1                                                */
/*  dosefile    Dose descriptor file name                                       */
/*  dosejump    Number of dose level increases allowed between consecutive      */
/*              patients                                                        */
/*  dtmodel     Dose-toxicity model, logisitic or Lyman                         */
/*  gamma       Gamma parameter for Lyman model                                 */
/*  level1      First dose level for trial                                      */
/*  maxsub      Maximum number of subjects in simulated trials                  */
/*  npat        Number of evaluable patients in single trial                    */
/*  nproceed    Number of patients needed to complete before dose escalation    */
/*  nrunin      Number of enrolled patients before stopping rule is checked     */
/*  nsim        Number of simulated trials                                      */
/*  obsdays     Number of days patient is followed for toxicity                 */
/*  patfile     Patient data input file name                                    */
/*  patform     Format type (1,2) of patient data                               */
/*  prior       Mean of normal prior                                            */
/*  priorstd    STD of normal prior                                             */
/*  priora      alpha of beta prior                                             */
/*  priorb      beta of beta prior                                              */
/*  readerr     Patient data read error flag, 0=No Error, 1=Error               */
/*  rectime     Mean interarrival time between patients                         */
/*  rscheme     Restriction scheme for dose-escalation                          */
/*              0=No restriction, use target dose                               */
/*              1=Allow only dosejump levels between patients                   */
/*              2=Require NPROCEED complete patients before escalation          */
/*              3=Reuqire NPROCEED observation periods between patients         */
/*              12=1+2                                                          */
/*              13=1+3                                                          */
/*  seed1       Random number seed for simulations                              */
/*  simflag     0=Single trial, 1=Simulate                                      */
/*  simtag      ID tag for simulation runs                                      */
/*  somedate    Fixed date (since 1/1/1960)                                     */
/*  target      Target toxicity probability                                     */
/*  td50        TD50 parameter for Lyman model                                  */
/*  tite        TITE-CRM flag =1 for TITE-CRM, 0 for CRM                        */
/*  today       Date of calculation                                             */
/*  vwflag      0=constant weighting throughout trial                           */
/*              1=variable weighting                                            */
/*  vwwts       w1...wk-1,1 for variable weighting                              */
/*                                                                              */
/********************************************************************************/

%macro simtite;

*------Print out debugging messages;

       %if &debug=1 %then %do;
           %put SIMTITE (ENTER) SIMFLAG &simflag;
           %put SIMTITE (ENTER) TITEFLAG &titeflag;
           %put SIMTITE (ENTER) OBSDAYS &obsdays;
           %if &simflag=0 %then %do;
               %put SIMTITE (ENTER) TODAY &today;
               %end;
           %if &simflag=1 %then %do;
               %put SIMTITE (ENTER) SEED1 &seed1;
               %end;
           %end;

*------Implement chosen dose-toxicity model;

       %if &dtmodel=logistic %then %do;
           %let dtcalc=%str(exp(3+alphahat*tdose)/(1+exp(3+alphahat*tdose)));
           %end;

       %if &dtmodel=lyman %then %do;
           %let dtcalc=%str(cdf('normal',(alphahat*tdose-td50)/gamma/td50,0,1));
           %end;

*------Get variable DLT weights and put them in a data set;

       %if &vwflag=1 %then %do;

           %let r=%scan(&vwdays,1,%str( ));
           %let vwn=1;
           %do %until(&r=);
               %let r=%scan(&vwdays,&vwn,%str( ));
               %let p=%scan(&vwwts,&vwn,%str( ));
               %if &r> %then %do;
                   %let vwday&vwn=&r;
                   %let vwwt&vwn=&p;
                   %let vwn=%eval(&vwn+1);
                   %end;
               %else %do;
                   %let vwn=%eval(&vwn-1);
                   %end;
               %end;

            data simtite1;
               %do i=1 %to &vwn;
                   vwday&i=input(symget("vwday&i"),4.);
                   vwwt&i=input(symget("vwwt&i"),5.3);
                  %end;
                 output;
                 stop;
                 keep vwday1-vwday&vwn vwwt1-vwwt&vwn;
                 run;

          %if &debug=1 %then %do;
              %put SIMTITE VWN &vwn;
              %do i=1 %to &vwn;
                  %put SIMTITE VWDAY&i &&&vwday&i   VWWT&i &&&vwwt&i;
                  %end;
              %end;

          %end;

*------Get variable DLT hazard weights and put them in a data set;

       %if &simflag=1 %then %do;

            libname thisdir "&outdir";

           %if &urflag=1 %then %do;

               %let r=%scan(&urdays,1,%str( ));
               %let urn=1;
               %do %until(&r=);
                   %let r=%scan(&urdays,&urn,%str( ));
                   %let p=%scan(&urwts,&urn,%str( ));
                   %if &r> %then %do;
                       %let urday&urn=&r;
                       %let urwt&urn=&p;
                       %let urn=%eval(&urn+1);
                       %end;
                   %else %do;
                       %let urn=%eval(&urn-1);
                       %end;
                   %end;

                data simtite2;
                   %do i=1 %to &urn;
                       urday&i=input(symget("urday&i"),4.);
                       urwt&i=input(symget("urwt&i"),5.3);
                      %end;
                     output;
                     stop;
                     keep urday1-urday&urn urwt1-urwt&urn;
                     run;

              %if &debug=1 %then %do;
                  %put SIMTITE URN &urn;
                  %do i=1 %to &urn;
                      %put SIMTITE URDAY&i &&&urday&i   URWT&i &&&urwt&i;
                      %end;
                  %end;

              %end;

*----------Calculate MAXSUB and NPER for variable recruitment plans;

           %if &varcruit=1 %then %do;

               %let r=%scan(&rectimes,1,%str( ));
               %let iper=1;
               %do %until(&r=);
                   %let r=%scan(&rectimes,&iper,%str( ));
                   %let p=%scan(&periods,&iper,%str( ));
                   %if &r> %then %do;
                       %let r&iper=&r;
                       %let p&iper=&p;
                       %let iper=%eval(&iper+1);
                       %end;
                   %else %do;
                       %let nper=%eval(&iper-1);
                       %end;
                   %end;

               data _null_;
                    array rectimes {*} r1-r&nper;
                    array periods {*} p1-p&nper;
                    nper=input(symget('nper'),2.);
                   %do i=1 %to &nper;
                       r&i=input(symget("r&i"),4.);
                       p&i=input(symget("p&i"),3.);
                      %end;
                    maxsub=0;
                    do i=1 to nper;
                       maxsub=maxsub+periods{i}/rectimes{i};
                       end;
                    call symput('maxsub',left(put(maxsub,3.)));
                    stop;
                    run;

               %put MAXSUB &maxsub;

               %end;

*----------Delete old simulation results before starting;

           %if &clearsim=1 %then %do;

               proc datasets library=thisdir;
                    delete patient trial / memtype=data;
                    run;
                    quit;

              %end;

           %end;     /*      end &simflag=1     */

*------Declare data sets, with different variables for simulated and real calculations;

       %if &simflag=0 %then %do;
           data patient(keep=id level response respdate dateon weight)
                   trial(keep=alphahat today newdose event1-event&nlevel
                               n1-n&nlevel p1-p&nlevel);
                length id $ 20;
           %end;

       %if &simflag=1 %then %do;
           data patient(keep=simtag isim level response respdate dateon alphahat)
                  trial(keep=simtag isim alphahat event1-event&nlevel
                             n1-n&nlevel p1-p&nlevel truep1-truep&nlevel final finishtrial);
                length simtag $ %length(&simtag)
                       rscheme $ 2;
           %end;

            length adm_dose $ 1;
            format dateon respdate date9.
                   level 2.0 response 1.;

            %if &simflag=0 %then %do;
                %put NPAT:  &npat;
                %let neval=&npat;
                %end;
            %if &simflag=1 %then %do;
                %put MAXSUB:  &maxsub;
                %let neval=&maxsub;
                %end;
            %put NLEVEL: &nlevel;

*-----------Patient-level arrays;

            array ons {*} dateon1-dateon&neval;
            array resps {*} response1-response&neval;
            array offs {*} dateoff1-dateoff&neval;
            array levs {*} lev1-lev&neval;
            array as {*} a1-a&neval;
            array ws {*} w1-w&neval;
            array xs {*} x1-x&neval;
            array ys {*} y1-y&neval;
            array ndays {*} nday1-nday&neval;

           %if &simflag=0 %then %do;
               array ids {*} $ 20 id1-id&neval;
              %end;

*-----------Dose-level arrays;

            array ts {*} t1-t&nlevel;
            array wts {*} wt1-wt&nlevel;
            array events {*} event1-event&nlevel;
            array ns {*} n1-n&nlevel;
            array ps {*} p1-p&nlevel;

*-----------For simulations, arrays of true probability of DLT and variable
            recruitment specifications;

           %if &simflag=1 %then %do;
               array trueps {*} truep1-truep&nlevel;
               %if &varcruit=1 %then %do;
                  array rectimes {*} r1-r&nper;
                  array periods {*} p1-p&nper;
                  %end;
              %end;

*----------Variable weighting arrays;

           %if &vwflag=1 %then %do;
               array vwdays {*} vwday1-vwday&vwn;
               array vwwts {*} vwwt1-vwwt&vwn;
               vwn=input(symget('vwn'),2.);
               do ivwn=1 to nvwn;
                  set simtite1 point=ivwn nobs=nvwn;
                  end;
               %end;

*----------Variable weighting arrays;

           %if &simflag=1 & &urflag=1 %then %do;
               array urdays {*} urday1-urday&urn;
               array urwts {*} urwt1-urwt&urn;
               urn=input(symget('urn'),2.);
               do iurn=1 to nurn;
                  set simtite2 point=iurn nobs=nurn;
                  end;
               %end;

*-----------Get current macro control variables;

            today=input(symget('today'),8.);
            obsdays=input(symget('obsdays'),3.);
            titeflag=input(symget('titeflag'),1.);
            nproceed=input(symget('nproceed'),2.);
            target=input(symget('target'),4.2);
            nlevel=input(symget('nlevel'),2.);
            nrunin=input(symget('nrunin'),2.);
            adm_dose=symget('adm_dose');
            adm_marg=input(symget('adm_marg'),4.2);
            rscheme=symget('rscheme');
            if (index(rscheme,'1') gt 0) then dosejump=input(symget('dosejump'),1.);

           %if &simflag=0 %then %do;
               iflag=input(symget('iflag'),1.);
               npat=input(symget('npat'),3.);
              %end;

           %if &simflag=1 %then %do;
               maxsub=input(symget('maxsub'),3.);
               nsim=input(symget('nsim'),4.);
               seed1=input(symget('seed1'),13.);
               rectime=input(symget('rectime'),5.);
               simtag=symget('simtag');
              %end;

           %if &prior=beta %then %do;
                priora=input(symget('priora'),9.);
                priorb=input(symget('priorb'),9.);
                %if &debug=1 %then %do;
                    put 'PRIORA:  ' priora /
                        'PRIORB:  ' priorb;
                    %end;
               %end;

           %if &prior=normal %then %do;
                priorstd=input(symget('priorstd'),9.);
                %if &debug=1 %then %do;
                    put 'PRIORSTD:  ' priorstd;
                    %end;
               %end;

           %if &dtmodel=lyman %then %do;
                gamma=input(symget('gamma'),6.);
                td50=input(symget('td50'),6.);
               %if &debug=1 %then %do;
                    put 'GAMMA:  ' gamma /
                        'TD50:  ' td50;
                    %end;
               %end;

           %if &debug=1 %then %do;
               put 'TODAY:  '   today /
                 'OBSDAYS:  '    obsdays /
                 'TITEFLAG:  ' titeflag /
                 'NPROCEED:  ' nproceed /
                 'TARGET:  '  target /
                 'RSCHEME:  ' rscheme ;
                if (index(rscheme,'1') gt 0) then put 'DOSEJUMP:  ' dosejump ;

               %if &simflag=0 %then %do;
                   put 'NPAT:  '    npat;
                  %end;
               %if &simflag=1 %then %do;
                   put 'SEED1:  ' seed1 /
                       'MAXSUB:  ' maxsub /
                       'NSIM:  '    nsim;
                  %end;
               %end;

*-----------Get the trial doses;

            do ilevel=1 to nlevel;
               set doses point=ilevel nobs=nlevel;
               ts{ilevel}=t;
              %if &simflag=1 %then %do;
                  trueps{ilevel}=truep;
                 %end;
               end;

           %if &debug=1 %then %do;
               %if &simflag=0 %then %do;
                   do ilevel=1 to nlevel;
                      put ilevel '   T:  ' ts{ilevel};
                      end;
                    %end;
               %if &simflag=1 %then %do;
                   do ilevel=1 to nlevel;
                      put ilevel '   T:  ' ts{ilevel}  '  TRUE P:  '  trueps{ilevel};
                      end;
                    %end;
                %end;

*----------Begin code for single-trial estimation from input data;

           %if &simflag=0 %then %do;

*--------------Off-study dates conform to the following rules

                  Response=0
                      If response date is greater than on-date+obsdays,
                           response date is set to on-date+obsdays
                      If response date is less than on-date+obsdays,
                           response date is unchanged

                  Response=1
                      Off date is fully specified

                  Response=2
                      Off date is fully specified

                  Response=.
                      If response date is missing, response is set to 0 and
                          response date is set to
                          min(today.on-date+obsedays)
                 ;

                  ipat=0;
                  do i=1 to nobs;

                     set arrive point=i nobs=nobs;

                     if (. le response le 1) or
                              ((response eq 2) and (iflag eq 1)) then do;
                        ipat+1;
                        ons{ipat}=dateon;
                        levs{ipat}=level;
                        ids{ipat}=id;
                        if (response eq .) then do;
                           resps{ipat}=0;
                           offs{ipat}=min(today,ons{ipat}+obsdays);
                           end;
                        if (response eq 0) then do;
                           resps{ipat}=0;
                           offs{ipat}=min(today,ons{ipat}+obsdays);
                           end;
                        if (response eq 1) then do;
                           resps{ipat}=1;
                           offs{ipat}=respdate;
                           end;
                        if ((response eq 2) and (iflag eq 1)) then do;
                           resps{ipat}=0;
                           offs{ipat}=respdate;
                           end;

                        ndays{ipat}=offs{ipat}-ons{ipat};

                       %if &debug=1 %then %do;
                           put ipat '  DATE ON:  ' ons{ipat} '  DATE OFF:  ' offs{ipat}
                                    '  RESPONSE:  ' resps{ipat};
                           %end;

                        end;
                     end;

                  npat=ipat;

                %if &debug=1 %then %do;
                    put 'EVALUABLE PATIENTS READ (NPAT):  ' npat;
                   %end;

                  link alphahat;

                  ipat=npat+1;
                  link setdose;

                %if &debug=1 %then %do;
                    put 'NEWDOSE:  ' newdose;
                   %end;

                  do ilevel=1 to nlevel;
                     ns{ilevel}=0;
                     events{ilevel}=0;
                     tdose=ts{ilevel};
                     ps{ilevel}=&dtcalc;
                     end;

                  do ipat=1 to npat;
                     ns{levs{ipat}}+1;
                     events{levs{ipat}}+resps{ipat};
                     dateon=ons{ipat};
                     respdate=offs{ipat};
                     level=levs{ipat};
                     response=resps{ipat};
                     id=ids{ipat};
                     weight=ws{ipat};
                     put ipat dateon respdate level response alphahat;
                     output patient;
                     end;

                  output trial;

                  stop;

                %end;   /*   end &simflag=0    */

                 %if &simflag=1 %then %do;

*--------------------Loop over the number of simulations;

                     do isim=1 to nsim;

*-----------------------Reset all patient-level data to missing;

                        do ipat=1 to maxsub;
                           ons{ipat}=.;
                           offs{ipat}=.;
                           resps{ipat}=.;
                           levs{ipat}=.;
                           as{ipat}=.;
                           ws{ipat}=.;
                           xs{ipat}=.;
                           ys{ipat}=.;
                           end;

*-----------------------Set the arrival times;

                        link sarrive;

*-----------------------Set the initial value of alpha;

                        %if &dtmodel=logistic %then %do;
                            meana=1;
                            %end;
                        %if &dtmodel=lyman %then %do;
                            %if &prior=beta %then %do;
                                meana=(priora-1)/(priora+priorb-2);
                               %end;
                            %end;

*-----------------------Loop through the patients.  Determine the dose, determine
                        the response, set the value of alpha;

                        finishtrial=1;

                        do ipat=1 to maxsub until(finishtrial eq 0);

*--------------------------If NEWDOSE is missing (first patient) or previous NEWDOSE was
                           greater than 0, find the dose, response and response date.  If
                           NEWDOSE is 0, dont calculate any more doses, and set everybodys
                           other values to missing;

                           if (ipat eq 1) then as{ipat}=meana;

                           link setdose;

                           %if &debug=1 %then %do;
                               put 'NEWDOSE:  ' newdose;
                              %end;

*--------------------------If NEWDOSE eqauals 0, this patient doesnt count;

                           if (newdose eq 0) then do;
                              finishtrial=0;
                              npat=ipat-1;
                              end;

                           if (newdose gt 0) then do;

                              levs{ipat}=newdose;
                              npat=ipat;

*-----------------------------Now, we know dose for patient IPAT.   Determine the
                              response and date off-study;

                              call ranbin(seed1,1,trueps{levs{ipat}},response);

                              if (response eq 1) then do;
                                 %if &urflag=0 %then %do;
                                     call ranuni(seed1,x);
                                     respdate=ons{ipat}+floor(obsdays*x);
                                    %end;
                                 %if &urflag=1 %then %do;
                                     call ranuni(seed1,x);
                                     if (x le urwt1) then do;
                                        respdate=ons{ipat}+floor(x*obsdays/urwt1);
                                        end;
                                     else do;
                                        do j=2 to urn until(urwts{j-1} lt x le urwts{j});
                                           end;
                                        respdate=ons{ipat}+urdays{j-1}+
                                            floor((urdays{j}-urdays{j-1})*(x-urwts{j-1})/
                                            (urwts{j}-urwts{j-1}));
                                        end;
                                    %end;
                                 end;

                              if (response eq 0) then do;
                                 respdate=ons{ipat}+obsdays;
                                 end;
                              resps{ipat}=response;
                              offs{ipat}=respdate;

*-----------------------------Set alphahat on the day the next patient enters, or
                              after the response of the final patient;

                              if (ipat lt maxsub) then today=ons{ipat+1};
                                 else today=max(of dateoff1-dateoff&maxsub)+1;

                              do j=1 to ipat;
                                 if (resps{j} eq 0) then
                                   ndays{j}=min(today,offs{j})-ons{j};
                                 if (resps{j} eq 1) and (offs{j} gt today) then
                                     ndays{j}=today-ons{j};
                                 if (resps{j} eq 1) and (offs{j} le today) then
                                     ndays{j}=offs{j}-ons{j};
                                 end;

*-----------------------------Calculate ALPHAHAT, given patient data and TODAY;

                              link alphahat;

                              if (ipat lt maxsub) then as{ipat+1}=alphahat;

                              %if &debug=1 %then %do;
                                  if (ipat lt maxsub) then put 'AS:  ' as{ipat+1};
                                  if (ipat eq maxsub) then put '  ALPHAHAT: ' alphahat;
                                  %end;

                              end;  /*  NEWDOSE > 0  */

*--------------------------End of IPAT loop;

                           end;

*------------------------End of ISIM loop after writing to data sets.  NPAT is actual number
                         of patients in trial (<=MAXSUB);

                         final=.;
                         dp=1;

                         if (finishtrial eq 0) then do;
                            final=0;
                            today=.;
                            do ipat=1 to npat;
                               today=max(today,offs{ipat});
                               end;
                            ipat=npat;
                            link alphahat;
                            end;

                         do ilevel=1 to nlevel;
                            ns{ilevel}=0;
                            events{ilevel}=0;
                            tdose=ts{ilevel};
                            ps{ilevel}=&dtcalc;
                            if (abs(ps{ilevel}-target) lt dp) then do;
                               dp=abs(ps{ilevel}-target);
                               if (finishtrial) then final=ilevel;
                               end;
                            end;

                         do ipat=1 to npat;
                            if (levs{ipat} gt 0) then do;
                               ns{levs{ipat}}+1;
                               events{levs{ipat}}+resps{ipat};
                               end;
                            end;

                         output trial;

                         do ipat=1 to npat;
                            if (levs{ipat} gt 0) then do;
                               alphahat=as{ipat};
                               dateon=ons{ipat};
                               respdate=offs{ipat};
                               level=levs{ipat};
                               response=resps{ipat};
                               output patient;
                               end;
                            end;

                         end;

*---------------------Leaving the DATA Step.  Put the seed back in the macro variable,
                      write the data set from the arrays;

                      call symput('seed1',left(put(seed1,13.)));

                      stop;

                     %end;         /*      end &simflag=1   */

*----------------Simulate arrival times;

                %if &simflag=1 %then %do;

                    sarrive:

                       %if &varcruit=1 & &titeflag=1 %then %do;

                           nper=input(symget('nper'),2.);
                           %do iper=1 %to &nper;
                               r&iper = input(symget("r&iper"),4.);
                               p&iper = input(symget("p&iper"),3.);
                              %end;
                           ipat=1;
                           ons{ipat}=input(symget('today'),8.);
                           do iper=1 to nper;
                              thissub=periods{iper}/rectimes{iper};
                              n1=ipat+1;
                              n2=ipat+thissub;
                              if (iper eq nper) then n2=maxsub;
                                 do ipat=n1 to n2;
                                 call ranuni(seed1,days);
                                 ons{ipat}=ons{ipat-1}+round(2*days*rectimes{iper},1);
                                 end;
                                 ipat=n2;
                              end;

                           %end;


                       %else %do;

                           %if &titeflag=0 %then %do;

                               ipat=1;
                               today=input(symget('today'),8.);
                               ons{ipat}=today;
                               do ipat=2 to maxsub;
                                  ons{ipat}=ons{ipat-1}+obsdays+1;
                                  end;

                               %end;


                           %if &titeflag=1 %then %do;

                               ipat=1;
                               today=input(symget('today'),8.);
                               ons{ipat}=today;
                               do ipat=2 to maxsub;
                                  call ranuni(seed1,days);
                                  ons{ipat}=ons{ipat-1}+round(2*days*rectime,1);
                                  end;

                               %end;

                           %end;

                    %if &debug=1 %then %do;
                        do ipat=1 to maxsub;
                           put ipat '  DAY ON STUDY:  ' ons{ipat};
                           end;
                         %end;

                     return;

                     %end;

*----------------Calculate ALPHAHAT, given TODAY, and IPAT patients, described by:

                     levs{i}    Treatment level
                     resps{i}   Response on day offs{i}
                     ons{i}     Day on trial
                     offs{i}    Day of recorded response

                     ;

                 alphahat:

                    %if &debug=1 %then %do;
                        put 'ALPHAHAT, IPAT:  ' ipat;
                       %end;

*-------------------xs{i} and ys{i} are the dose and response at the time of the
                      calculation,
                    ws{i} is the weighting;

                    do i=1 to ipat;
                       xs{i}=ts{levs{i}};

*----------------------ys{i} is set to 1 only if the response occurred before today;

                       ys{i}=0;
                       if (resps{i} eq 1) and (offs{i} le today) then ys{i}=1;

*----------------------Completed and DLT patients are fully weighted;

                       if (ys{i} eq 1) then ws{i}=1;
                       if (ys{i} eq 0) and (ndays{i} eq obsdays) then ws{i}=1;

*----------------------On default, set weights to CRM estimator;

                       if (ys{i} eq 0) and (ons{i} le today lt offs{i}) then ws{i}=0;

*----------------------If TITE-CRM flag equals 1, use TITE-CRM weights;

                      %if &vwflag=0 %then %do;
                          if (titeflag) and (ys{i} eq 0) and (ndays{i} lt obsdays) then
                                ws{i}=ndays{i}/obsdays;
                           %end;

                      %if &vwflag=1 %then %do;
                          if (titeflag) and (ys{i} eq 0) and (ndays{i} lt obsdays)
                              then do;
                             if (ndays{i} le vwday1) then do;
                                 ws{i}=vwwt1*ndays{i}/vwday1;
                                 end;
                             else do;
                                do j=2 to vwn until(vwdays{j-1} lt ndays{i} le
                                                        vwdays{j});
                                   end;
                                ws{i}=vwwts{j-1}+(vwwts{j}-vwwts{j-1})*
                                        (ndays{i}-vwdays{j-1})/
                                        (vwdays{j}-vwdays{j-1});
                                end;
                             end;
                          %end;

                       %if &debug=1 %then %do;
                            put i 'XS:  ' xs{i} '  YS:  ' ys{i} '  NDAYS:  '
                                 ndays{i} '  WS:  ' ws{i};
                           %end;

                       end;

*-------------------Determine points of numerical integration on A axis;

                    alo=0;
                    ahi=100;
                    nint=1000*ahi;

*-------------------Calculate numerator and denominator integrals by trapezoidal rule;

                    num=0;
                    denom=0;
                    lasta=alo;
                   %if &prior=beta %then %do;
                       lastb=pdf('beta',lasta,priora,priorb);
                      %end;
                   %if &prior=exp %then %do;
                       lastb=exp(-lasta);
                      %end;
                   %if &prior=normal %then %do;
                       lastb=pdf('normal',lasta,1,priorstd);
                      %end;

*-------------------Initialize the trapezoidal rule summand;

                    do j=1 to ipat;

*----------------------Standard logistic model;

                      %if &dtmodel=logistic %then %do;
                           psi=ws{j}*exp(3+lasta*xs{j})/(1+exp(3+lasta*xs{j}));
                          %end;

*----------------------Lyman model;

                       %if &dtmodel=lyman %then %do;
                           psi=ws{j}*cdf('normal',(lasta*xs{j}-td50)/gamma/td50,0,1);
                          %end;

                       if (ys{j} eq 0) then lastb=lastb*(1-psi);
                          else lastb=lastb*psi;
                       end;

                    lastc=lasta*lastb;
                    sflag=0;
                    lalphahat=-10;

*-------------------Integrate the functions until the ratio does not change significantly;

                    do i=1 to nint until(sflag eq 1);
                       a=alo+i*(ahi-alo)/nint;
                      %if &prior=beta %then %do;
                          b=pdf('beta',a,priora,priorb);
                         %end;
                      %if &prior=exp %then %do;
                          b=exp(-a);
                         %end;
                      %if &prior=normal %then %do;
                          b=pdf('normal',a,1,priorstd);
                         %end;
                      do j=1 to ipat;
                         %if &dtmodel=logistic %then %do;
                             psi=ws{j}*exp(3+a*xs{j})/(1+exp(3+a*xs{j}));
                            %end;
                         %if &dtmodel=lyman %then %do;
                             psi=ws{j}*cdf('normal',(a*xs{j}-td50)/gamma/td50,0,1);
                            %end;
                          if (ys{j} eq 0) then b=b*(1-psi);
                             else b=b*psi;
                          end;
                       c=a*b;
                       num=num+(a-lasta)*(c+lastc)/2;
                       denom=denom+(a-lasta)*(b+lastb)/2;
                       lasta=a; lastb=b; lastc=c;

                       if (mod(i,1000) eq 0) then do;
                          alphahat=num/denom;
                          if (abs(alphahat-lalphahat) le 1e-8) then sflag=1;
                          lalphahat=alphahat;
                          end;

                       end;

                    %if &debug=1 %then %do;
                        put 'ALPHAHAT:  ' alphahat '   UPPER A:  ' a  '   I:  ' I;
                        %end;


                    return;

*--------------------------------------------------------------------------------;

                setdose:

*-----------------First patient;

                  if (ipat eq 1) then do;
                      levs{ipat}=input(symget('level1'),1.);
                      lastlev=levs{ipat};
                      newdose=levs{ipat};
                      end;

*-----------------Later patients;

                  else do;

*---------------------Calculate the unadjusted target dose;

                      newdose=0;
                      minmarg=10;

                      do ilevel=1 to nlevel;
                         wts{ilevel}=0;
                         tdose=ts{ilevel};
                         ps{ilevel}=&dtcalc;

                         if (adm_dose eq '1') then do;
                            if (ps{ilevel} le target) then newdose=ilevel;
                            end;

                         if (adm_dose eq '2') then do;
                            if (abs(ps{ilevel}-target) lt minmarg) and (ps{ilevel} le target+adm_marg) then do;
                               minmarg=abs(ps{ilevel}-target);
                               newdose=ilevel;
                               end;
                            end;

                        %if &debug=1 %then %do;
                            put 'PS:  ' ps{ilevel};
                           %end;

                         end;

                     %if &debug=1 %then %do;
                         put 'NEWDOSE BEFORE ESC RULES:  ' newdose;
                        %end;


                      if (newdose eq 0) and (ipat le nrunin) and (p1 le target+adm_marg) then newdose=1;

                      lastlev=levs{ipat-1};

*---------------------Adjust NEWDOSE if it goes up more than one level or
                      there is insufficient data to advance;

                      if (newdose gt lastlev) then do;

*------------------------If RSCHEME=2 or 3, loop through patients to check that we
                         have enough experience to advance dose;

                         if (index(rscheme,'2') gt 0) or (index(rscheme,'3') gt 0) then
                             do;

                            do i=1 to ipat-1;

*------------------------------Always count a patient with a DLT already occured as a
                               full weight.  Count a patient with a later event as
                               partial under RSCHEME=3;

                               if (resps{i} eq 1) then do;

                                  if (offs{i} le today) then wts{levs{i}}+1;
                                  if (offs{i} gt today) and (index(rscheme,'3') gt 0)
                                      then wts{levs{i}}+ws{i};
                                  end;

*------------------------------Count a patient without a DLT already occured as a full
                               weight if observation period is complete under
                               RSCHEME 2 or 3.  Count a patient who has not completed
                               as partial under RSCHEME=3;

                               if (resps{i} eq 0) then do;
                                  if (today ge ons{i}+obsdays) then wts{levs{i}}+1;
                                  if (ons{i} le today lt ons{i}+obsdays) and
                                      (index(rscheme,'3') gt 0) then
                                         wts{levs{i}}+ws{i};
                                  end;
                               end;   *---do i=1 to ipat-1;
                            end;   *---RSCHEME= 2 or 3;

*------------------------Evaluate escalation rules;

                         if (index(rscheme,'1') gt 0) and (newdose gt lastlev+dosejump)
                             then newdose=lastlev+dosejump;

                         if ((index(rscheme,'2') gt 0) or (index(rscheme,'3') gt 0))
                              and (wts{lastlev} lt nproceed) then newdose=lastlev;

                         end;   *---newdose>lastlev;

                      end;   *----ipat>1;

                    return;

                   run;


*-----------------End of Main Data Step;


      %if &simflag=0 %then %do;

          proc sort data=doses;
               by level;
               run;

          data simtite6;
               length d1-d&nlevel $ 20;
               retain d1-d&nlevel ' '
                      p0_1-p0_&nlevel .;
               array doses {*} d1-d&nlevel;
               array ps {*} p1-p&nlevel;
               array p0s {*} p0_1-p0_&nlevel;
               array ns {*} n1-n&nlevel;
               array events {*} event1-event&nlevel;
               do i=1 to ndoses;
                  set doses point=i nobs=ndoses;
                  doses{i}=dose;
                  p0s{i}=p;
                  end;
               i=1;
               set trial point=i;
               do level=1 to ndoses;
                  p=ps{level};
                  dose=doses{level};
                  n=ns{level};
                  event=events{level};
                  p0=p0s{level};
                  output;
                  end;
               call symput('thedate',put(date(),date9.));
               stop;
               keep level dose p p0 n event;
               label p='P(Tox)';
               run;

          data _null_;
               set trial;
               call symput('alphahat',put(alphahat,5.3));
               run;

          data _null_;
               do i=1 to ntrial;
                  set trial point=i nobs=ntrial;
                  end;
               set doses point=newdose;
               call symput('nextdose',trim(dose));
               stop;
               run;

         %if &title1^=%str(Example Trial) %then %do;
              ods pdf file="&title1._&thedate..pdf";
             %end;
         %else %do;
              ods pdf file="titecrm_&thedate..pdf";
             %end;

        %if &iflag=0 %then %do;
             proc format;
                  value codes 0='0=No DLT'
                              1='1=DLT'
                              2='2=NE'
                              3='3=NE';
                  run;
             %end;

        %if &iflag=1 %then %do;
             proc format;
                  value codes 0='0=No DLT'
                              1='1=DLT'
                              2='2=No DLT, Partial'
                              3='3=NE';
                  run;
             %end;


          options pageno=1;

          proc print uniform label data=simtite6;
               label level='Level'
                     dose='Dose'
                     p=' Posterior P(DLT)'
                     event='# DLTs'
                     n='# Treated'
                     p0='Prior P(DLT)';
               id level;
               var dose p event n p0;
               title1 "&title1";
               title2 "Estimated Toxicities Based on Data Set &patfile";
               footnote1 "Target Rate is &target";
               footnote2 "Estimated alpha is &alphahat";
               footnote3 "Next Assigned Dose is &nextdose";
               footnote4 " ";
               footnote5 "Source:  titecrm.sas &srcdate";
               run;

          proc sort data=all;
               by id;
               run;

          proc sort data=patient;
               by id;
               run;

          data all2;
               merge all patient(in=in1 keep=id weight respdate rename=(respdate=rpd));
               by id;
               if (1-in1) then weight=0;
               if (in1) then do;
                  if (response eq .) then do;
                     response=0;
                     respdate=rpd;
                     end;
                  end;
               run;

          proc sort data=all2;
               by dateon;
               run;

          proc print uniform noobs label data=all2;
               label dateon='On Study'
                     id='Reg #'
                     dose='Dose'
                     level='Level'
                     respdate='Response Date'
                     response='Response'
                     weight='Weight';
               format response codes. weight 4.2;
               var id dateon dose level respdate response weight;
               title2 "Data Set &patfile";
               footnote1 "Source:  titecrm.sas &srcdate";
               run;
               quit;

          ods pdf close;

         %end;

      %if &simflag=1 %then %do;

          proc append force base=thisdir.patient data=patient;
               run;

          proc append force base=thisdir.trial data=trial;
               run;

         %end;

      %if &debug=0 %then %do;
          proc datasets library=work;
               delete simtite6 / memtype=data;
               run;
               quit;
          %if &vwflag=1 %then %do;
              proc datasets library=work;
                   delete simtite1 / memtype=data;
                   run;
                   quit;
              %end;
          %end;

      %if &debug=1 %then %do;
          %put SIMTITE (EXIT) SEED1 &seed1;
          %end;

      %mend simtite;

/********************************************************************************/
/*                                                                              */
/*  The following invokes the above macros.  See DEFAULTS macro for most of the */
/*  macro variables that are set.  The variables below are sufficiently         */
/*  important that you have to know what they mean.                             */
/*                                                                              */
/********************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Toxicity estimation from a trial example (README Mode 1) with fixed hazard   */
/* rate (uniform weighting)                                                     */
/*                                                                              */
/********************************************************************************/

%defaults

%let title1=%str(SINGLE TRIAL EXAMPLE);
%let simflag=0;
%let dosefile=%str(example_doses.dat);
%let obsdays=126;
%let level1=2;
%let target=0.15;

%let rscheme=13;
%let nproceed=2;
%let iflag=1;

%let vwflag=0;

%let patfile=%str(example_patients_1.csv);

%dday

%defdose

%patdata

%simtite


/********************************************************************************/
/*                                                                              */
/* Toxicity estimation from a trial example (README Mode 1) with variable       */
/* hazard                                                                       */
/*                                                                              */
/********************************************************************************/

%defaults

%let title1=%str(SINGLE TRIAL EXAMPLE WITH VARIABLE HAZARD);
%let simflag=0;
%let dosefile=%str(example_doses.dat);
%let obsdays=126;
%let level1=2;
%let target=0.15;

%let rscheme=13;
%let nproceed=2;
%let iflag=1;

%let vwflag=1;
%let vwdays=%str(21 42 63 84 105 126);
%let vwwts=%str(0.3 0.5 0.65 0.8 0.9 1.0);

%let patfile=%str(example_patients_1.csv);

%dday

%defdose

%patdata

%simtite

/********************************************************************************/
/*                                                                              */
/* Simulation example (README Mode 2), select dose LE target, 5 patient run-in  */
/*                                                                              */
/********************************************************************************/


%defaults

%let clearsim=1;

%let simflag=1;
%let debug=1;
%let dosefile=%str(sim2_doses.dat);
%let obsdays=90;
%let level1=2;
%let target=0.25;
%let nrunin=5;

%let varcruit=0;
%let maxsub=30;
%let rectime=30;

%let rscheme=13;
%let nproceed=1;
%let iflag=1;

%let vwflag=0;
%let urflag=0;

%let seed1=3948573;
%let nsim=2;
%let simtag=sim1a;

%dday

%defdose

%simtite

/********************************************************************************/
/*                                                                              */
/* Simulation example (README Mode 2), dose closest to target, 0.10 max         */
/* overdose                                                                     */
/*                                                                              */
/********************************************************************************/


%defaults

%let clearsim=0;

%let simflag=1;
%let debug=1;
%let dosefile=%str(sim2_doses.dat);
%let obsdays=90;
%let level1=2;
%let target=0.25;

%let varcruit=0;
%let maxsub=30;
%let rectime=30;

%let rscheme=13;
%let nproceed=1;
%let iflag=1;

%let vwflag=0;
%let urflag=0;

%let seed1=3948573;
%let nsim=2;
%let simtag=sim1b;

%let adm_dose=2;
%let adm_marg=0.10;

%dday

%defdose

%simtite


/********************************************************************************/
/*                                                                              */
/* Print out simulation results                                                 */
/*                                                                              */
/********************************************************************************/

ods pdf file='sim_results.pdf';

proc print uniform data=current.patient;
     title 'Patient-Level Simulation Results';
     run;

proc print uniform data=current.trial;
     title 'Trial-Level Simulation Results';
     run;
     quit;

ods pdf close;
